/**
 * Copyright (c) 2022 Yansen Zhang
 * wxcomponent is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
**/

package authorizer

import (
	"encoding/json"
	"errors"
	"net/url"

	wxerr "gitee.com/yansen_zh/wxcomponent/errors"
	"gitee.com/yansen_zh/wxcomponent/utils/request"
)

// GetAuthorizerOptionParam 获取授权方选项信息参数
type GetAuthorizerOptionParam struct {
	ComponentAppId  string `json:"component_appid"`
	AuthorizerAppId string `json:"authorizer_appid"`
	OptionName      string `json:"option_name"`
}

// GetAuthorizerOptionResult 获取授权方选项信息结果
type GetAuthorizerOptionResult struct {
	wxerr.Error
	AuthorizerAppId string `json:"authorizer_appid"`
	OptionName      string `json:"option_name"`
	OptionValue     string `json:"option_value"`
}

const (
	apiGetAuthorizerOption = "https://apies.weixin.qq.com/cgi-bin/component/api_get_authorizer_option"
)

// GetAuthorizerOption 获取授权方选项信息
func GetAuthorizerOption(componentAccessToken string, data GetAuthorizerOptionParam) (*GetAuthorizerOptionResult, error) {
	if componentAccessToken == "" {
		return nil, errors.New("获取授权方选项信息 第三方平台的 component_access_token 不能为空")
	}

	if data.ComponentAppId == "" {
		return nil, errors.New("获取授权方选项信息 授权公众号或小程序的 appid 不能为空")
	}

	if data.AuthorizerAppId == "" {
		return nil, errors.New("获取授权方选项信息 第三方平台的 appid 不能为空")
	}

	if data.OptionName == "" {
		return nil, errors.New("获取授权方选项信息 选项名称 不能为空")
	}

	queryParam := url.Values{}
	queryParam.Set("component_access_token", componentAccessToken)

	resp, e2 := request.PostJSON(apiGetAuthorizerOption, &queryParam, data)
	if e2 != nil {
		return nil, e2
	}

	result := GetAuthorizerOptionResult{}
	if err := json.Unmarshal(resp, &result); err != nil {
		return nil, err
	}

	return &result, nil
}
